package cn.com.qmth.stmms.biz.mark.service.Impl;

import cn.com.qmth.stmms.biz.exam.dao.*;
import cn.com.qmth.stmms.biz.exam.model.*;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.mark.dao.*;
import cn.com.qmth.stmms.biz.mark.model.*;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.biz.utils.TaskLock;
import cn.com.qmth.stmms.biz.utils.TaskLockUtil;
import cn.com.qmth.stmms.common.enums.*;
import cn.com.qmth.stmms.common.utils.BigDecimalUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 与评卷相关的所有修改操作(非传播性的新增操作除外)，全部汇总到这里进行集中控制
 *
 * @author luoshi
 */
@Service("markService")
public class MarkServiceImpl implements MarkService {

    protected static final Logger log = LoggerFactory.getLogger(MarkServiceImpl.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private MarkGroupDao groupDao;

    @Autowired
    private MarkGroupStudentDao groupStudentDao;

    @Autowired
    private SubjectiveScoreDao scoreDao;

    @Autowired
    private ExamQuestionDao questionDao;

    @Autowired
    private MarkerDao markerDao;

    @Autowired
    private MarkLibraryDao libraryDao;

    @Autowired
    private ArbitrateHistoryDao arbitrateDao;

    @Autowired
    private MarkTrackDao trackDao;

    @Autowired
    private MarkSpecialTagDao specialTagDao;

    @Autowired
    private TrialLibraryDao trialLibraryDao;

    @Autowired
    private TrialHistoryDao trialHistoryDao;

    @Autowired
    private TrialTrackDao trialTrackDao;

    @Autowired
    private TrialTagDao trialTagDao;

    @Autowired
    private ProblemHistoryDao problemHistoryDao;

    @Autowired
    private ExamService examService;

    @Autowired
    private ExamQuestionService questionService;

    private Map<Integer, Long> markerLastUpdateTime = new ConcurrentHashMap<>();

    /**
     * 某个评卷分组已申请的评卷任务数量
     *
     * @param group
     *            - 评卷分组
     * @return int
     */
    @Override
    public int applyCount(MarkGroup group) {
        int count = 0;
        if (group.getStatus() == MarkStatus.TRIAL) {
            TaskLock taskLock = TaskLockUtil.getTrialTask(getGroupKey(group));
            count = taskLock.count();
        } else if (group.getStatus() == MarkStatus.FORMAL) {
            TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(group));
            count = taskLock.count();
        }
        return count;
    }

    /**
     * 某个评卷员已申请的评卷任务数量
     *
     * @param marker
     *            - 评卷员
     * @return int
     */
    @Override
    public int applyCount(Marker marker) {
        MarkGroup group = marker.getGroup();
        if (group == null) {
            group = groupDao.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        }
        if (group != null) {
            if (group.getStatus() == MarkStatus.FORMAL) {
                TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(marker));
                return taskLock.count(marker.getId());
            } else if (group.getStatus() == MarkStatus.TRIAL) {
                TaskLock taskLock = TaskLockUtil.getTrialTask(getGroupKey(marker));
                return taskLock.count(marker.getId());
            }
        }
        return 0;
    }

    /**
     * 某个评卷员已完成的评卷任务数量
     *
     * @param marker
     *            - 评卷员
     * @return long
     */
    @Override
    public long markedCount(Marker marker) {
        MarkGroup group = marker.getGroup();
        if (group == null) {
            group = groupDao.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        }
        if (group != null) {
            if (group.getStatus() == MarkStatus.TRIAL) {
                return trialHistoryDao.countByMarkerId(marker.getId());
            } else {
                return libraryDao.countByMarkerAndStatus(marker.getId(), LibraryStatus.MARKED,
                        LibraryStatus.ARBITRATED, LibraryStatus.INSPECTED);
            }
        }
        return 0;
    }

    /**
     * 释放某个评卷分组的锁定任务
     *
     * @param group
     *            - 评卷分组
     */
    @Override
    public void releaseByGroup(MarkGroup group) {
        if (group.getStatus() == MarkStatus.TRIAL) {
            TaskLock taskLock = TaskLockUtil.getTrialTask(getGroupKey(group));
            taskLock.clear();
        } else if (group.getStatus() == MarkStatus.FORMAL) {
            TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(group));
            taskLock.clear();
        }
    }

    /**
     * 重置某个评卷分组的所有评卷任务
     *
     * @param group
     *            - 评卷分组
     */
    @Override
    @Transactional
    public void resetByGroup(MarkGroup group) {
        resetGroup(group);
    }

    /**
     * 删除某个评卷分组
     *
     * @param group
     *            - 评卷分组
     */
    @Override
    @Transactional
    public void deleteGroup(MarkGroup group) {
        // 试评相关数据
        trialTrackDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        trialTagDao.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
        trialHistoryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        trialLibraryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        // 正评相关数据
        trackDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        specialTagDao.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
        arbitrateDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        problemHistoryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        libraryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        // 评卷员数据
        markerDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        // 小题数据
        questionDao.resetByExamIdAndSubjectCodeAndObjectiveAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                false, group.getNumber());
        // 考生分组状态与得分明细
        groupStudentDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        scoreDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                group.getNumber());
        // 删除分组
        releaseByGroup(group);
        groupDao.delete(group);
        // 未分组的题目
        long unGroupQuestionCount = questionDao.countByExamIdAndSubjectAndObjectiveAndGroupNumberIsNull(
                group.getExamId(), group.getSubjectCode(), false);
        // 考生整体状态与总分更新
        long groupCount = groupDao.countByExamIdAndSubjectCode(group.getExamId(), group.getSubjectCode());
        if (groupCount == 0 || unGroupQuestionCount > 0) {
            studentService.updateSubjectiveStatusAndScoreAndInspectorId(group.getExamId(), group.getSubjectCode(),
                    SubjectiveStatus.UNMARK, 0, null, null, null);
        } else {
            List<Integer> studentList = studentService.findIdByExamIdAndSubjectCodeAndSubjectiveStatus(
                    group.getExamId(), group.getSubjectCode(), SubjectiveStatus.UNMARK, SubjectiveStatus.MARKED,
                    SubjectiveStatus.INSPECTED);
            for (Integer studentId : studentList) {
                checkStudentSubjective(studentId, groupCount, unGroupQuestionCount);
            }
        }
    }

    /**
     * 修改某个评卷分组给分步骤，并重置评卷任务
     *
     * @param group
     *            - 评卷分组
     */
    @Override
    @Transactional
    public void updateGroup(MarkGroup group, List<ExamQuestion> questionList, ScorePolicy policy, ThirdPolicy third) {
        List<ExamQuestion> old = questionDao.findByExamIdAndSubjectCodeAndObjectiveAndGroupNumber(group.getExamId(),
                group.getSubjectCode(), false, group.getNumber());
        for (ExamQuestion question : old) {
            question.setGroupNumber(null);
            questionDao.saveAndFlush(question);
        }
        double totalScore = 0d;
        for (ExamQuestion question : questionList) {
            totalScore += question.getTotalScore();
            question.setGroupNumber(group.getNumber());
            questionDao.saveAndFlush(question);
        }
        groupDao.updateTotalScore(group.getExamId(), group.getSubjectCode(), group.getNumber(), totalScore);
        groupDao.updateScorePolicy(group.getExamId(), group.getSubjectCode(), group.getNumber(), policy);
        groupDao.updateThirdPolicy(group.getExamId(), group.getSubjectCode(), group.getNumber(), third);
        subjectService.updateScore(group.getExamId(), group.getSubjectCode(), false,
                sumTotalScore(group.getExamId(), group.getSubjectCode()));
        resetGroup(group);
        if (group.getStatus() == MarkStatus.FORMAL) {
            libraryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
        }
        if (group.getStatus() == MarkStatus.TRIAL) {
            trialLibraryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
        }
        updateLibraryCount(group);
        groupDao.updateBuildTime(group.getExamId(), group.getSubjectCode(), group.getNumber(), null);
    }

    /**
     * 评卷员申请领取某个正式评卷任务
     *
     * @param library
     *            - 正评任务
     * @param marker
     *            - 评卷员
     */
    @Override
    public boolean applyLibrary(MarkLibrary library, Marker marker) {
        // 查询待领取任务时，已经做了多评同一studentId互斥处理
        TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(marker));
        boolean lock = taskLock.add(library.getStudentId(), library.getTaskNumber(), marker.getId());
        // 上锁失败直接返回
        if (!lock) {
            return false;
        }
        // 重复校验任务状态
        if (libraryDao.countByIdAndStatus(library.getId(), library.getStatus()) == 1) {
            return true;
        } else {
            taskLock.remove(library.getStudentId(), library.getTaskNumber(), marker.getId());
            return false;
        }
    }

    /**
     * 评卷员申请领取某个试评评卷任务
     *
     * @param library
     *            - 试评任务
     * @param marker
     *            - 评卷员
     * @return boolean
     */
    @Override
    public boolean applyLibrary(TrialLibrary library, Marker marker) {
        TaskLock taskLock = TaskLockUtil.getTrialTask(getGroupKey(marker));
        return taskLock.add(library.getStudentId(), 1, marker.getId());
    }

    /**
     * 评卷员是否已领取了某个正式评卷任务
     *
     * @param library
     *            - 正评任务
     * @param marker
     *            - 评卷员
     * @return boolean
     */
    @Override
    public boolean hasApplied(MarkLibrary library, Marker marker) {
        TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(marker));
        return taskLock.exist(library.getStudentId(), library.getTaskNumber(), marker.getId());
    }

    /**
     * 评卷员是否已领取了某个试评评卷任务
     *
     * @param library
     *            - 试评任务
     * @param marker
     *            - 评卷员
     * @return boolean
     */
    @Override
    public boolean hasApplied(TrialLibrary library, Marker marker) {
        TaskLock taskLock = TaskLockUtil.getTrialTask(getGroupKey(marker));
        return taskLock.exist(library.getStudentId(), 1, marker.getId());
    }

    /**
     * 释放某个评卷员已完成的评卷任务
     *
     * @param result
     *            - 评卷结果
     * @param marker
     *            - 评卷员
     */
    @Override
    public void releaseTask(SubmitResult result, Marker marker) {
        if (result.getMarkLibrary() != null) {
            TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(marker));
            taskLock.remove(result.getMarkLibrary().getStudentId(), result.getMarkLibrary().getTaskNumber(),
                    marker.getId());
            taskLock.refresh(marker.getId());
        } else if (result.getTrialLibrary() != null) {
            TaskLock taskLock = TaskLockUtil.getTrialTask(getGroupKey(marker));
            taskLock.remove(result.getTrialLibrary().getStudentId(), 1, marker.getId());
        }
    }

    /**
     * 释放某个评卷员的所有锁定任务
     *
     * @param marker
     *            - 评卷员
     */
    @Override
    public void releaseByMarker(Marker marker) {
        TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(marker));
        taskLock.clear(marker.getId());
        TaskLock lock = TaskLockUtil.getTrialTask(getGroupKey(marker));
        lock.clear(marker.getId());
    }

    /**
     * 重置某个评卷员
     *
     * @param marker
     *            - 评卷员
     */
    @Override
    @Transactional
    public void resetMarker(Marker marker) {
        MarkGroup group = groupDao.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        if (group == null || !markerDao.exists(marker.getId())) {
            return;
        }
        if (group.getStatus() == MarkStatus.FORMAL) {
            // 仲裁和等待仲裁的任务不被重置
            // trackDao.deleteByMarkerId(marker.getId(),
            // LibraryStatus.ARBITRATED, LibraryStatus.WAIT_ARBITRATE,
            // LibraryStatus.PROBLEM);
            // specialTagDao.deleteByMarkerId(marker.getId(),
            // LibraryStatus.ARBITRATED, LibraryStatus.WAIT_ARBITRATE,
            // LibraryStatus.PROBLEM);
            // libraryDao.resetByMarkerId(marker.getId(), LibraryStatus.WAITING,
            // LibraryStatus.ARBITRATED,
            // LibraryStatus.WAIT_ARBITRATE, LibraryStatus.PROBLEM);
            // 原遍历模式改为SQL批量更新模式，提高执行速度
            // groupStudentDao.updateStatusByMarkLibraryStatus(marker.getExamId(),
            // marker.getSubjectCode(),
            // marker.getGroupNumber(), SubjectiveStatus.UNMARK,
            // LibraryStatus.WAITING);
            // scoreDao.deleteByStudentGroupStatus(marker.getExamId(),
            // marker.getSubjectCode(), marker.getGroupNumber(),
            // SubjectiveStatus.UNMARK);
            // studentService.resetSubjectiveStatusByGroupStatus(marker.getExamId(),
            // marker.getSubjectCode());
            // 只选取评卷完成状态的记录重新检查
            // List<Integer> studentIdList =
            // groupStudentDao.findStudentIdByGroupNumberAndStatus(marker.getExamId(),
            // marker.getSubjectCode(), marker.getGroupNumber(),
            // SubjectiveStatus.MARKED);
            // for (Integer studentId : studentIdList) {
            // checkStudentGroup(studentId, group);
            // }
            // 遍历相关评卷任务的模式
            List<MarkLibrary> list = libraryDao.findByMarkerIdAndStatusNotIn(marker.getId(), LibraryStatus.ARBITRATED,
                    LibraryStatus.WAIT_ARBITRATE, LibraryStatus.PROBLEM);
            for (MarkLibrary library : list) {
                trackDao.deleteByLibraryId(library.getId());
                specialTagDao.deleteByLibraryId(library.getId());
                libraryDao.resetById(library.getId(), null, LibraryStatus.WAITING, library.getStatus());
                updateStudentGroupStatus(library.getStudentId(), library.getExamId(), library.getSubjectCode(),
                        library.getGroupNumber(), SubjectiveStatus.UNMARK);
                studentService.updateSubjectiveStatusAndScore(library.getStudentId(), SubjectiveStatus.UNMARK, 0, null);
            }
            markerDao.resetById(marker.getId());
        } else if (group.getStatus() == MarkStatus.TRIAL) {
            trialTagDao.deleteByMarkerId(marker.getId());
            trialTrackDao.deleteByMarkerId(marker.getId());
            trialHistoryDao.deleteByMarkerId(marker.getId());
        }
        updateMarkedCount(group);
        releaseByMarker(marker);
    }

    /**
     * 根据考生删除评卷任务
     *
     * @param student
     *            - 考生
     */
    @Override
    @Transactional
    public void deleteByStudent(ExamStudent student) {
        // 正评相关数据
        trackDao.deleteByStudentId(student.getId());
        specialTagDao.deleteByStudentId(student.getId());
        arbitrateDao.deleteByStudentId(student.getId());
        problemHistoryDao.deleteByStudentId(student.getId());
        libraryDao.deleteByStudentId(student.getId());
        // 试评相关数据
        trialTrackDao.deleteByStudentId(student.getId());
        trialTagDao.deleteByStudentId(student.getId());
        trialHistoryDao.deleteByStudentId(student.getId());
        trialLibraryDao.deleteByStudentId(student.getId());
        // 主观状态与得分明细
        groupStudentDao.deleteByStudentId(student.getId());
        scoreDao.deleteByStudentId(student.getId());
        updateAllCount(student.getExamId(), student.getSubjectCode());
    }

    /**
     * 评卷员提交评卷任务
     *
     * @param result
     *            - 评卷结果
     * @param marker
     *            - 评卷员
     * @return boolean
     */
    @Override
    @Transactional
    public SubmitResult submitTask(MarkResult result, Marker marker) {
        // 判断评卷分组是否存在/评卷是否结束
        MarkGroup group = groupDao.findOne(marker.getExamId(), marker.getSubjectCode(), marker.getGroupNumber());
        if (group == null || group.getStatus() == MarkStatus.FINISH
                || !group.getStatus().toString().equals(result.getStatusValue())) {
            return SubmitResult.faile();
        }

        // 根据评卷状态选择读取不同的评卷任务
        if (group.getStatus() == MarkStatus.FORMAL) {
            MarkLibrary library = libraryDao.findOne(result.getLibraryId());
            if (library != null && result.isProblem()) {
                // 状态更新
                Date now = new Date();
                if (libraryDao.updateProblemResult(library.getId(), LibraryStatus.PROBLEM, marker.getId(), now,
                        result.getSpent(), LibraryStatus.WAITING, LibraryStatus.MARKED, LibraryStatus.INSPECTED,
                        LibraryStatus.REJECTED) != 0) {
                    saveProblemHistory(result, library);
                    updateMarkedCount(group);
                    // 未评完
                    resetStudentGroup(library.getStudentId(), library.getExamId(), library.getSubjectCode(),
                            library.getGroupNumber());
                    trackDao.deleteByLibraryId(library.getId());
                    specialTagDao.deleteByLibraryId(library.getId());
                    return SubmitResult.success(library);
                }
            }
            if (library != null && library.getExamId().equals(group.getExamId())
                    && library.getSubjectCode().equals(group.getSubjectCode())
                    && library.getGroupNumber().equals(group.getNumber())
                    && result.getMarkerScore() <= group.getTotalScore()
                    && StringUtils.isNotBlank(result.getScoreList())) {
                if (submitLibrary(library, marker, group, result)) {
                    updateMarkedCount(group);
                    return SubmitResult.success(library);
                }
            }
        } else if (group.getStatus() == MarkStatus.TRIAL) {
            TrialLibrary library = trialLibraryDao.findOne(result.getLibraryId());
            if (library != null && library.getExamId().equals(group.getExamId())
                    && library.getSubjectCode().equals(group.getSubjectCode())
                    && library.getGroupNumber().equals(group.getNumber())
                    && result.getMarkerScore() <= group.getTotalScore()
                    && StringUtils.isNotBlank(result.getScoreList())) {
                TrialHistory history = new TrialHistory();
                history.setExamId(library.getExamId());
                history.setSubjectCode(library.getSubjectCode());
                history.setGroupNumber(library.getGroupNumber());
                history.setLibraryId(library.getId());
                history.setStudentId(library.getStudentId());
                history.setMarkerId(marker.getId());
                history.setMarkerTime(new Date());
                history.setMarkerScore(result.getMarkerScore());
                history.setMarkerScoreList(result.getScoreList());
                history.setSecretNumber(library.getSecretNumber());
                history = trialHistoryDao.save(history);
                if (history != null) {
                    if (result.getTrackList() != null) {
                        trialTrackDao.deleteByLibraryIdAndMarkerId(history.getLibraryId(), history.getMarkerId());
                        trialTrackDao.save(result.getTrackList(history));
                    }
                    if (result.getSpecialTagList() != null) {
                        trialTagDao.deleteByLibraryIdAndMarkerId(history.getLibraryId(), history.getMarkerId());
                        trialTagDao.save(result.getTagList(history));
                    }
                    updateMarkedCount(group);
                    // releaseLibrary(library, marker);
                    return SubmitResult.success(library);
                }
            }
        }
        return SubmitResult.faile();
    }

    private void saveProblemHistory(MarkResult result, MarkLibrary library) {
        ProblemHistory history = problemHistoryDao.findByLibraryIdAndStatus(library.getId(), HistoryStatus.WAITING);
        if (history == null) {
            history = new ProblemHistory();
        }
        history.setCreateTime(new Date());
        history.setExamId(library.getExamId());
        history.setExamNumber(library.getExamNumber());
        history.setSecretNumber(library.getSecretNumber());
        history.setStudentId(library.getStudentId());
        history.setLibraryId(library.getId());
        history.setSubjectCode(library.getSubjectCode());
        history.setGroupNumber(library.getGroupNumber());
        history.setProblemId(result.getProblemTypeId());
        history.setStatus(HistoryStatus.WAITING);
        problemHistoryDao.save(history);
    }

    /**
     * 评卷员提交某个正评任务
     *
     * @param library
     *            - 正评任务
     * @param marker
     *            - 评卷员
     * @param group
     *            - 评卷分组
     * @param result
     *            - 评卷结果
     */
    private boolean submitLibrary(MarkLibrary library, Marker marker, MarkGroup group, MarkResult result) {
        // 非本人领取的待评任务
        if ((library.getStatus() == LibraryStatus.WAITING || library.getStatus() == LibraryStatus.REJECTED)
                && !hasApplied(library, marker)) {
            return false;
        }
        // 非本人的回评任务
        if ((library.getStatus() == LibraryStatus.MARKED || library.getStatus() == LibraryStatus.INSPECTED)
                && !library.getMarkerId().equals(marker.getId())) {
            return false;
        }
        // 是否多评情况下已处理过该考生评卷任务
        if (libraryDao
                .countByStudentIdAndMarkerIdAndIdNotEqual(library.getStudentId(), marker.getId(), library.getId()) > 0) {
            return false;
        }
        // 尝试提交评卷结果
        Date now = new Date();
        if (libraryDao.updateMarkerResult(library.getId(), LibraryStatus.MARKED, marker.getId(),
                result.getMarkerScore(), result.getScoreList(), now, result.getSpent(), LibraryStatus.WAITING,
                LibraryStatus.MARKED, LibraryStatus.INSPECTED, LibraryStatus.REJECTED) == 0) {
            // 条件不符更新失败，直接返回
            return false;
        }
        // 保存阅卷轨迹
        if (result.getTrackList() != null) {
            trackDao.deleteByLibraryId(library.getId());
            List<MarkTrack> tracks = result.getTrackList(library, marker);
            for (MarkTrack markTrack : tracks) {
                trackDao.saveAndFlush(markTrack);
            }
        }
        // 保存特殊标记
        if (result.getSpecialTagList() != null) {
            specialTagDao.deleteByLibraryId(library.getId());
            specialTagDao.save(result.getSpecialTagList(library, marker));
        }
        // 判断多评模式下是否需要仲裁
        ArbitrateHistory history = null;
        if (group.getArbitrateThreshold() != null && group.getArbitrateThreshold() > 0) {
            // 多评模式
            List<MarkLibrary> list = libraryDao.findByStudentIdAndGroupNumberAndStatus(library.getStudentId(),
                    library.getGroupNumber(), LibraryStatus.MARKED, LibraryStatus.INSPECTED);
            for (MarkLibrary other : list) {
                if (other.getId().equals(library.getId()) || other.getHeaderScore() != null) {
                    // 本评卷任务或组长已打分，则跳过该任务
                    continue;
                }
                // 分差超过阀值，
                if (Math.abs(other.getMarkerScore() - result.getMarkerScore()) > group.getArbitrateThreshold()) {
                    // 开启三评
                    if (group.getThirdPolicy().equals(ThirdPolicy.LOW_DIFF_HIGH_AVG)) {
                        if (libraryDao.countByStudentIdAndGroupNumber(library.getStudentId(), library.getGroupNumber()) == 2) {
                            buildThirdLibrary(library, group);
                        } else {
                            // 两两比较，触发仲裁
                            List<MarkLibrary> libraries = libraryDao.findByStudentIdAndGroupNumber(
                                    library.getStudentId(), library.getGroupNumber());
                            history = buildArbitrateHistory(libraries, group.getArbitrateThreshold(), now);
                        }
                    } else {
                        // 未开启三评，触发仲裁
                        history = buildArbitrateHistory(library, now);
                        break;
                    }
                }
            }
        }
        if (history != null) {
            // 保存仲裁记录
            arbitrateDao.save(history);
            // 触发仲裁后续状态更新
            libraryDao.updateByStudentIdAndGroupNumber(library.getStudentId(), library.getGroupNumber(),
                    LibraryStatus.WAIT_ARBITRATE);
            // 未评完
            resetStudentGroup(library.getStudentId(), library.getExamId(), library.getSubjectCode(),
                    library.getGroupNumber());
        } else {
            // 判断当前分组是否已完成评卷
            checkStudentGroup(library.getStudentId(), group);
        }
        return true;
    }

    private ArbitrateHistory buildArbitrateHistory(List<MarkLibrary> list, Double arbitrateThreshold, Date now) {
        for (MarkLibrary library : list) {
            if (library.getStatus() != LibraryStatus.MARKED && library.getStatus() != LibraryStatus.ARBITRATED
                    && library.getStatus() != LibraryStatus.INSPECTED) {
                return null;
            }
        }
        int count = 0;
        for (int i = 0; i < list.size(); i++) {
            int next = i + 1;
            if (next == list.size()) {
                next = 0;
            }
            if (Math.abs(list.get(i).getMarkerScore() - list.get(next).getMarkerScore()) > arbitrateThreshold) {
                count++;
            }
        }
        if (count == list.size()) {
            return buildArbitrateHistory(list.get(0), now);
        }
        return null;
    }

    private ArbitrateHistory buildArbitrateHistory(MarkLibrary library, Date now) {
        ArbitrateHistory history = new ArbitrateHistory();
        history.setExamId(library.getExamId());
        history.setSubjectCode(library.getSubjectCode());
        history.setGroupNumber(library.getGroupNumber());
        history.setStudentId(library.getStudentId());
        history.setExamNumber(library.getExamNumber());
        history.setSecretNumber(library.getSecretNumber());
        history.setStatus(HistoryStatus.WAITING);
        history.setCreateTime(now);
        return history;
    }

    private void buildThirdLibrary(MarkLibrary markLibrary, MarkGroup group) {
        MarkLibrary library = new MarkLibrary();
        library.setExamId(markLibrary.getExamId());
        library.setSubjectCode(markLibrary.getSubjectCode());
        library.setGroupNumber(group.getNumber());
        library.setStudentId(markLibrary.getStudentId());
        library.setExamNumber(markLibrary.getExamNumber());
        library.setSecretNumber(markLibrary.getSecretNumber());
        library.setTaskNumber(3);
        library.setStatus(LibraryStatus.WAITING);
        libraryDao.save(library);
        updateLibraryCount(group);
    }

    /**
     * 管理员/组长打回某个评卷任务<br>
     * isRest为true时重置该评卷任务
     *
     * @param library
     *            - 正评任务
     */
    @Override
    @Transactional
    public boolean rejectLibrary(MarkLibrary library, Integer userId, boolean isRest) {
        MarkGroup group = groupDao.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());
        if (group.getStatus() == MarkStatus.FINISH) {
            return false;
        }
        if (libraryDao.resetById(library.getId(), null, isRest ? LibraryStatus.WAITING : LibraryStatus.REJECTED,
                LibraryStatus.MARKED, LibraryStatus.PROBLEM, LibraryStatus.INSPECTED) > 0) {
            trackDao.deleteByLibraryId(library.getId());
            specialTagDao.deleteByLibraryId(library.getId());
            resetStudentGroup(library.getStudentId(), library.getExamId(), library.getSubjectCode(),
                    library.getGroupNumber());
            problemHistoryDao.resetByLibraryId(library.getId(), HistoryStatus.WAITING, userId, HistoryStatus.BACK,
                    new Date());
            // 开启三评时，打回1，2任务则删除第3条任务
            long count = libraryDao.countByStudentIdAndGroupNumber(library.getStudentId(), library.getGroupNumber());
            if (library.getTaskNumber() != 3 && count == 3) {
                MarkLibrary third = libraryDao.findByStudentIdAndGroupNumberAndTaskNumber(library.getStudentId(),
                        library.getGroupNumber(), 3);
                trackDao.deleteByLibraryId(third.getId());
                specialTagDao.deleteByLibraryId(third.getId());
                problemHistoryDao.deleteByLibraryId(third.getId());
                libraryDao.delete(third);
                updateLibraryCount(group);
            }
            updateMarkedCount(group);
            return true;
        } else {
            return false;
        }
    }

    /**
     * 管理员/组长重置某个试评任务
     *
     * @param library
     *            - 试评任务
     */
    @Override
    @Transactional
    public boolean resetLibrary(TrialLibrary library) {
        if (library != null) {
            trialHistoryDao.deleteByLibraryId(library.getId());
            trialTrackDao.deleteByLibraryId(library.getId());
            trialTagDao.deleteByLibraryId(library.getId());
            return true;
        } else {
            return false;
        }
    }

    /**
     * 管理员/组长处理仲裁卷
     *
     * @param history
     *            - 仲裁卷
     */
    @Override
    @Transactional
    public void processArbitrate(ArbitrateHistory history) {
        if (arbitrateDao.exists(history.getId())) {
            arbitrateDao.saveAndFlush(history);
            libraryDao.updateHeaderResult(history.getStudentId(), history.getGroupNumber(), history.getUserId(),
                    history.getTotalScore(), history.getScoreList(), history.getUpdateTime(), LibraryStatus.ARBITRATED);
            updateMarkedCount(history.getExamId(), history.getSubjectCode(), history.getGroupNumber());
            checkStudentGroup(history.getStudentId(),
                    groupDao.findOne(history.getExamId(), history.getSubjectCode(), history.getGroupNumber()));
        }
    }

    // /**
    // * 对某个考生某个科目进行主观题统分
    // *
    // * @param student - 考生
    // * @param groups - 所有评卷分组
    // */
    // @Override
    // @Transactional
    // public void scoreCalculate(ExamStudent student, List<MarkGroup> groups) {
    // if (student == null || groups == null || groups.isEmpty()) {
    // return;
    // }
    // List<ScoreItem> scoreList = new ArrayList<>();
    // double totalScore = 0.0;
    // // 循环所有评卷分组
    // for (MarkGroup group : groups) {
    // if (group.getStatus().equals(MarkStatus.TRIAL)) {
    // return;
    // }
    // if (calculateGroup(group, student.getId())) {
    // totalScore += group.getMarkScore();
    // scoreList.addAll(group.getMarkScoreDetail());
    // } else {
    // // 未评完直接返回
    // return;
    // }
    // }
    // // 全部评完，更新考生主观题得分
    // student.setSubjectiveScore(totalScore);
    // student.setScoreList(scoreList, false);
    // studentService
    // .updateSubjectiveScore(student.getId(), student.getSubjectiveScore(),
    // student.getSubjectiveScoreList());
    // }

    private void scoreCalculate(Integer studentId) {
        List<ScoreItem> scoreList = new ArrayList<>();
        Map<Integer, List<SubjectiveScore>> map = new HashMap<Integer, List<SubjectiveScore>>();
        double totalScore = 0.0;
        // 循环所有主观得分明细
        List<SubjectiveScore> list = scoreDao.findByStudentId(studentId);
        for (SubjectiveScore ss : list) {
            totalScore = BigDecimalUtils.add(totalScore, ss.getScore());
            scoreList.add(new ScoreItem(ss));
            List<SubjectiveScore> mainScoreList = map.get(ss.getMainNumber());
            if (mainScoreList == null) {
                mainScoreList = new ArrayList<SubjectiveScore>();
            }
            mainScoreList.add(ss);
            map.put(ss.getMainNumber(), mainScoreList);
        }
        // 计算大题分
        for (Integer mainNumber : map.keySet()) {
            List<SubjectiveScore> mainScoreList = map.get(mainNumber);
            double mainScore = 0.0;
            for (SubjectiveScore subjectiveScore : mainScoreList) {
                mainScore = BigDecimalUtils.add(mainScore, subjectiveScore.getScore());
            }
            for (SubjectiveScore subjectiveScore : mainScoreList) {
                subjectiveScore.setMainScore(mainScore);
                scoreDao.saveAndFlush(subjectiveScore);
            }
        }
        // 全部评完，更新考生主观题得分
        studentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.MARKED, totalScore,
                ExamStudent.buildScoreList(scoreList));
    }

    private boolean calculateGroup(MarkGroup group, Integer studentId) {
        double score = 0;
        List<ScoreItem> detail = new ArrayList<>();
        int count = 0;
        // 未设置算分策略的情况下，默认取平均分
        ScorePolicy policy = group.getScorePolicy() != null ? group.getScorePolicy() : ScorePolicy.AVG;
        List<MarkLibrary> list = libraryDao.findByStudentIdAndGroupNumber(studentId, group.getNumber());
        if (list.isEmpty()) {
            return false;
        }
        for (MarkLibrary library : list) {
            if (library.getStatus() != LibraryStatus.MARKED && library.getStatus() != LibraryStatus.ARBITRATED
                    && library.getStatus() != LibraryStatus.INSPECTED) {
                // 有非完成状态的评卷任务，直接返回
                return false;
            }
        }
        // 开启三评
        if (ThirdPolicy.LOW_DIFF_HIGH_AVG.equals(group.getThirdPolicy()) && list.size() == 3) {
            // Collections.sort(list,
            // Comparator.comparing(MarkLibrary::getMarkerScore));
            // 从小到大排序
            list.sort((o1, o2) -> {
                if (o1.getMarkerScore() - o2.getMarkerScore() > 0) {
                    return 1;
                }
                return -1;
            });
            Double score1 = list.get(0).getHeaderScore() != null ? list.get(0).getHeaderScore() : list.get(0)
                    .getMarkerScore();
            Double score2 = list.get(1).getHeaderScore() != null ? list.get(1).getHeaderScore() : list.get(1)
                    .getMarkerScore();
            Double score3 = list.get(2).getHeaderScore() != null ? list.get(2).getHeaderScore() : list.get(2)
                    .getMarkerScore();
            if ((score3 - score2) <= (score2 - score1)) {
                list.remove(0);
            } else {
                list.remove(2);
            }
            policy = ScorePolicy.AVG;
        }
        for (MarkLibrary library : list) {
            count++;
            Double current = (library.getHeaderScore() != null ? library.getHeaderScore() : library.getMarkerScore());
            List<ScoreItem> scores = library.getScoreList();
            if (count == 1) {
                // 首份评卷任务，直接取总分与明细
                score = current;
                detail = scores;
            } else {
                switch (policy) {
                case AVG:
                    // 直接累加
                    score += current;
                    for (int i = 0; i < detail.size(); i++) {
                        try {
                            ScoreItem item = detail.get(i);
                            ScoreItem other = scores.get(i);
                            item.setScore(BigDecimalUtils.add(item.getScore(), other.getScore()));
                        } catch (Exception e) {
                            continue;
                        }
                    }
                    break;
                case MAX:
                    // 高分优先
                    if (current > score) {
                        score = current;
                        detail = scores;
                    }
                    break;
                case MIN:
                    // 低分优先
                    if (current < score) {
                        score = current;
                        detail = scores;
                    }
                    break;
                default:
                    break;
                }
            }
        }
        // 取平均分策略下，累计分数需要重新计算一次
        if (policy == ScorePolicy.AVG && count > 1) {
            score = score / count;
            for (ScoreItem item : detail) {
                item.setScore(item.getScore() / count);
            }
        }
        group.setMarkScore(score);
        group.setMarkScoreDetail(detail);
        return true;
    }

    private void updateMarkedCount(Integer examId, String subjectCode, Integer groupNumber) {
        MarkGroup group = groupDao.findOne(examId, subjectCode, groupNumber);
        if (group != null) {
            updateMarkedCount(group);
        }
    }

    /**
     * 更新某个评卷分组已评任务数量
     *
     * @param group
     *            - 评卷分组
     */
    @Override
    @Transactional
    public void updateMarkedCount(MarkGroup group) {
        if (group.getStatus() == MarkStatus.FORMAL || group.getStatus() == MarkStatus.FINISH) {
            groupDao.updateMarkedCount(group.getExamId(), group.getSubjectCode(), group.getNumber(), (int) libraryDao
                    .countByExamIdAndSubjectCodeAndGroupNumberAndStatus(group.getExamId(), group.getSubjectCode(),
                            group.getNumber(), LibraryStatus.MARKED, LibraryStatus.ARBITRATED, LibraryStatus.INSPECTED));
        } else if (group.getStatus() == MarkStatus.TRIAL) {
            groupDao.updateMarkedCount(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                    (int) trialLibraryDao.countMarked(group.getExamId(), group.getSubjectCode(), group.getNumber()));
        }
    }

    /**
     * 更新某个评卷分组评卷任务总量
     *
     * @param group
     *            - 评卷分组
     */
    @Override
    @Transactional
    public void updateLibraryCount(MarkGroup group) {
        if (group.getStatus() == MarkStatus.FORMAL || group.getStatus() == MarkStatus.FINISH) {
            group.setLibraryCount((int) libraryDao.countByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(),
                    group.getSubjectCode(), group.getNumber()));
            groupDao.updateLibraryCount(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                    group.getLibraryCount());
        } else if (group.getStatus() == MarkStatus.TRIAL) {
            group.setLibraryCount((int) trialLibraryDao.countByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(),
                    group.getSubjectCode(), group.getNumber()));
            groupDao.updateLibraryCount(group.getExamId(), group.getSubjectCode(), group.getNumber(),
                    group.getLibraryCount());
        }
    }

    /**
     * 更新某个科目所有评卷分组评卷任务数量
     *
     * @param examId
     *            - 考试ID
     * @param subjectCode
     *            - 科目代码
     */
    @Transactional
    @Override
    public void updateAllCount(Integer examId, String subjectCode) {
        List<MarkGroup> groups = groupDao.findByExamIdAndSubjectCode(examId, subjectCode);
        for (MarkGroup group : groups) {
            updateMarkedCount(group);
            updateLibraryCount(group);
        }
    }

    /**
     * 根据考生、学习中心、评卷分组构造正式评卷任务
     *
     * @param student
     *            - 考生
     * @param group
     *            - 评卷分组
     * @param subject
     *            - 科目
     */
    @Override
    @Transactional
    public void buildFormalLibrary(ExamStudent student, MarkGroup group, ExamSubject subject) {
        // 查询是否已创建评卷任务
        if (libraryDao.countByStudentIdAndGroupNumber(student.getId(), group.getNumber()) == 0) {
            MarkLibrary library = new MarkLibrary();
            library.setExamId(student.getExamId());
            library.setSubjectCode(student.getSubjectCode());
            library.setGroupNumber(group.getNumber());
            library.setStudentId(student.getId());
            library.setExamNumber(student.getExamNumber());
            library.setSecretNumber(student.getSecretNumber());
            library.setTaskNumber(1);
            library.setStatus(LibraryStatus.WAITING);
            libraryDao.save(library);
            group.setLibraryCount(group.getLibraryCount() + 1);
            // 开启双评时需要判断是否生成第二份评卷任务
            if (group.getDoubleRate() != null && group.getDoubleRate() > 0) {
                boolean needDouble = false;
                if (group.getDoubleRate() == 1) {
                    needDouble = true;
                } else {
                    double studentCount = subject.getUploadCount();
                    double libraryCount = group.getLibraryCount();
                    double doubleCount = libraryDao.countByExamIdAndSubjectCodeAndGroupNumberAndTaskNumber(
                            group.getExamId(), group.getSubjectCode(), group.getNumber(), 2);
                    int expectCount = (int) (studentCount * group.getDoubleRate());
                    // 随机数判断加入当前已经生成双评任务的比例加权
                    // 实际双评任务数小于理论生成数 &&（剩余未生成双评的考生数量小于剩余应生成的数量||随机比例）
                    needDouble = doubleCount < expectCount
                            && ((studentCount - libraryCount + doubleCount) <= (expectCount - doubleCount) || Math
                                    .random() < group.getDoubleRate() + 0.1);
                }
                if (needDouble) {
                    library = new MarkLibrary();
                    library.setExamId(student.getExamId());
                    library.setSubjectCode(student.getSubjectCode());
                    library.setGroupNumber(group.getNumber());
                    library.setStudentId(student.getId());
                    library.setExamNumber(student.getExamNumber());
                    library.setSecretNumber(student.getSecretNumber());
                    library.setTaskNumber(2);
                    library.setStatus(LibraryStatus.WAITING);
                    libraryDao.save(library);
                    group.setLibraryCount(group.getLibraryCount() + 1);

                }
            }
        }
        group.setBuildTime(student.getUploadTime());
        groupDao.updateBuildTime(group.getExamId(), group.getSubjectCode(), group.getNumber(), student.getUploadTime());
    }

    /**
     * 根据考生、学习中心、评卷分组构造试评评卷任务
     *
     * @param student
     *            - 考生
     * @param group
     *            - 评卷分组
     */
    @Override
    @Transactional
    public void buildTrialLibrary(ExamStudent student, MarkGroup group) {
        // 查询是否已创建评卷任务
        if (trialLibraryDao.countByStudentIdAndGroupNumber(student.getId(), group.getNumber()) == 0) {
            TrialLibrary library = new TrialLibrary();
            library.setExamId(student.getExamId());
            library.setSubjectCode(student.getSubjectCode());
            library.setGroupNumber(group.getNumber());
            library.setStudentId(student.getId());
            library.setExamNumber(student.getExamNumber());
            library.setSecretNumber(student.getSecretNumber());
            trialLibraryDao.save(library);
        }
    }

    private String getGroupKey(MarkGroup group) {
        return group.getExamId() + "_" + group.getSubjectCode() + "_" + group.getNumber();
    }

    private String getGroupKey(Marker marker) {
        return marker.getExamId() + "_" + marker.getSubjectCode() + "_" + marker.getGroupNumber();
    }

    /**
     * 重置评卷分组的连带操作
     *
     * @param group
     *            - 评卷分组
     */
    private void resetGroup(MarkGroup group) {
        if (group.getStatus() == MarkStatus.FORMAL) {
            trackDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
            specialTagDao.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
            arbitrateDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
            problemHistoryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
            libraryDao.resetByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber(), LibraryStatus.WAITING);
            libraryDao.deleteByExamIdAndSubjectCodeAndGroupNumberAndTaskNumber(group.getExamId(),
                    group.getSubjectCode(), group.getNumber(), 3);
            markerDao.resetByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
            resetStudentGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
        } else if (group.getStatus() == MarkStatus.TRIAL) {
            trialTrackDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
            trialTagDao.deleteByExamAndSubjectAndGroup(group.getExamId(), group.getSubjectCode(), group.getNumber());
            trialHistoryDao.deleteByExamIdAndSubjectCodeAndGroupNumber(group.getExamId(), group.getSubjectCode(),
                    group.getNumber());
        }
        updateLibraryCount(group);
        groupDao.resetCount(group.getExamId(), group.getSubjectCode(), group.getNumber());
        releaseByGroup(group);
    }

    /**
     * 计算并更新指定评卷员的评卷质量指标
     *
     * @param marker
     *            - 评卷员
     */
    @Override
    @Transactional
    public void updateQuality(Marker marker) {
        List<MarkLibrary> list = libraryDao.findByMarkerId(marker.getId());
        int finishCount = 0;
        int validCount = 0;
        double sumScore = 0;
        double sumScore2 = 0;
        double avgScore = 0;
        double stdevScore = 0;
        double sumSpent = 0;
        double avgSpent = 0;
        for (MarkLibrary library : list) {
            finishCount++;
            if (library.getStatus() == LibraryStatus.MARKED || library.getStatus() == LibraryStatus.INSPECTED) {
                validCount++;
            }
            double score = library.getMarkerScore() != null ? library.getMarkerScore() : 0;
            int spent = library.getMarkerSpent() != null ? library.getMarkerSpent() : 0;

            sumScore += score;
            sumScore2 += Math.pow(score, 2);
            sumSpent += spent;
            if (finishCount > 0) {
                avgScore = sumScore / finishCount;
                avgSpent = sumSpent / finishCount;
                // 递归法计算标准差
                stdevScore = Math.sqrt(sumScore2 / finishCount - Math.pow(sumScore / finishCount, 2));
            }
        }
        markerDao.updateQualityById(marker.getId(), finishCount, validCount, avgSpent / 1000, avgScore, stdevScore);
        markerLastUpdateTime.put(marker.getId(), System.currentTimeMillis());
    }

    private double sumTotalScore(Integer examId, String subjectCode) {
        Double score = groupDao.sumTotalScore(examId, subjectCode);
        return score != null ? score : 0d;
    }

    @Override
    public boolean needUpdateQuality(Marker marker, long expireMinutes) {
        if (marker == null) {
            return false;
        }
        Long lastUpdateTime = markerLastUpdateTime.get(marker.getId());
        MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
        query.setPageNumber(1);
        query.setPageSize(1);
        query.setSort(new Sort(Sort.Direction.DESC, "markerTime"));
        List<MarkLibrary> list = libraryDao.findByMarkerIdAndStatus(marker.getId(), LibraryStatus.MARKED, query);
        MarkLibrary library = list.isEmpty() ? null : list.get(0);
        if (library != null && library.getMarkerTime() != null) {
            long lastMarkTime = library.getMarkerTime().getTime();
            if (lastUpdateTime != null && lastUpdateTime > lastMarkTime) {
                return false;
            } else {
                return (System.currentTimeMillis() - lastMarkTime) < (expireMinutes * 60 * 1000);
            }
        }
        return false;
    }

    /**
     * 考生判断是否主观题全部评卷完成
     *
     * @param studentId
     */
    private void checkStudentSubjective(Integer studentId, long groupCount, long unGroupQuestionCount) {
        if (groupStudentDao.countByStudentIdAndStatus(studentId, SubjectiveStatus.MARKED) == groupCount
                && unGroupQuestionCount == 0) {
            scoreCalculate(studentId);
        } else {
            studentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0, null);
        }
    }

    /**
     * 考生分组判断是否评卷完成，以及后续的统一处理动作
     *
     * @param studentId
     * @param group
     */
    private void checkStudentGroup(Integer studentId, MarkGroup group) {
        if (calculateGroup(group, studentId)) {
            updateStudentGroupStatus(studentId, group.getExamId(), group.getSubjectCode(), group.getNumber(),
                    SubjectiveStatus.MARKED);
            updateStudentGroupScore(studentId, group.getExamId(), group.getSubjectCode(), group.getNumber(),
                    group.getMarkScore(), group.getMarkScoreDetail());
            // 未分组的题目
            long unGroupQuestionCount = questionDao.countByExamIdAndSubjectAndObjectiveAndGroupNumberIsNull(
                    group.getExamId(), group.getSubjectCode(), false);
            long groupCount = groupDao.countByExamIdAndSubjectCode(group.getExamId(), group.getSubjectCode());
            checkStudentSubjective(studentId, groupCount, unGroupQuestionCount);
        } else {
            updateStudentGroupStatus(studentId, group.getExamId(), group.getSubjectCode(), group.getNumber(),
                    SubjectiveStatus.UNMARK);
            studentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0, null);
        }
    }

    /**
     * 重置考生某个分组的评卷状态，以及整体的主观题状态
     *
     * @param studentId
     * @param examId
     * @param subjectCode
     * @param groupNumber
     */
    private void resetStudentGroup(Integer studentId, Integer examId, String subjectCode, Integer groupNumber) {
        updateStudentGroupStatus(studentId, examId, subjectCode, groupNumber, SubjectiveStatus.UNMARK);
        scoreDao.deleteByStudentIdAndGroupNumber(studentId, groupNumber);
        studentService.updateSubjectiveStatusAndScore(studentId, SubjectiveStatus.UNMARK, 0, null);
    }

    /**
     * 重置某个分组所有考生的评卷状态，以及这些考生的主观题状态
     *
     * @param examId
     * @param subjectCode
     * @param groupNumber
     */
    private void resetStudentGroup(Integer examId, String subjectCode, Integer groupNumber) {
        groupStudentDao.updateStatusByExamIdAndSubjectCodeAndGroupNumber(examId, subjectCode, groupNumber,
                SubjectiveStatus.UNMARK);
        scoreDao.deleteByExamIdAndSubjectCodeAndGroupNumber(examId, subjectCode, groupNumber);
        studentService.updateSubjectiveStatusAndScoreAndInspectorId(examId, subjectCode, SubjectiveStatus.UNMARK, 0,
                null, null, null);
    }

    private void updateStudentGroupStatus(Integer studentId, Integer examId, String subjectCode, Integer groupNumber,
            SubjectiveStatus status) {
        MarkGroupStudent gs = new MarkGroupStudent();
        gs.setStudentId(studentId);
        gs.setExamId(examId);
        gs.setSubjectCode(subjectCode);
        gs.setGroupNumber(groupNumber);
        gs.setStatus(status);
        groupStudentDao.saveAndFlush(gs);
    }

    private void updateStudentGroupScore(Integer studentId, Integer examId, String subjectCode, Integer groupNumber,
            double score, List<ScoreItem> scoreList) {
        // scoreDao.deleteByStudentIdAndGroupNumber(studentId, groupNumber);
        List<ExamQuestion> questions = questionDao.findByExamIdAndSubjectCodeAndObjectiveAndGroupNumber(examId,
                subjectCode, false, groupNumber);
        for (int i = 0; i < scoreList.size(); i++) {
            ScoreItem item = scoreList.get(i);
            ExamQuestion question = questions.get(i);
            SubjectiveScore ss = new SubjectiveScore();
            ss.setStudentId(studentId);
            ss.setExamId(examId);
            ss.setSubjectCode(subjectCode);
            ss.setGroupNumber(groupNumber);
            ss.setGroupScore(score);
            ss.setMainScore(0.0);
            ss.setMainNumber(question.getMainNumber());
            ss.setSubNumber(question.getSubNumber());
            ss.setScore(item.getScore());
            scoreDao.saveAndFlush(ss);
        }
    }

    @Override
    @Transactional
    public boolean backStudentByGroup(ExamStudent student, Integer groupNumber, Integer userId) {
        MarkGroup group = groupDao.findOne(student.getExamId(), student.getSubjectCode(), groupNumber);
        if (group.getStatus() == MarkStatus.FINISH) {
            return false;
        }
        List<MarkLibrary> list = libraryDao.findByStudentIdAndGroupNumber(student.getId(), groupNumber);
        int count = 0;
        for (MarkLibrary library : list) {
            if (libraryDao.resetById(library.getId(), null, LibraryStatus.REJECTED, LibraryStatus.MARKED,
                    LibraryStatus.PROBLEM, LibraryStatus.INSPECTED) > 0) {
                count++;
                trackDao.deleteByLibraryId(library.getId());
                specialTagDao.deleteByLibraryId(library.getId());
                problemHistoryDao.resetByLibraryId(library.getId(), HistoryStatus.WAITING, userId, HistoryStatus.BACK,
                        new Date());
                if (library.getTaskNumber() == 3) {
                    // 出现三评时删除第三条任务
                    problemHistoryDao.deleteByLibraryId(library.getId());
                    libraryDao.delete(library);
                    updateLibraryCount(group);
                }
            }
        }
        if (count > 0) {
            updateMarkedCount(group);
            resetStudentGroup(student.getId(), group.getExamId(), group.getSubjectCode(), group.getNumber());
            studentService.updateSubjectiveStatusAndTimeAndInspectorId(student.getId(), SubjectiveStatus.UNMARK, null,
                    null);
            return true;
        }
        return false;
    }

    @Override
    @Transactional
    public boolean rejectedStudent(ExamStudent student, MarkStepDTO[] markStepDTOs, Integer userId) {
        Map<Integer, List<MarkStepDTO>> map = new HashMap<Integer, List<MarkStepDTO>>();

        for (MarkStepDTO markStepDTO : markStepDTOs) {
            List<MarkStepDTO> list = map.get(markStepDTO.getGroupNumber());
            if (list == null) {
                list = new ArrayList<MarkStepDTO>();
            }
            list.add(markStepDTO);
            map.put(markStepDTO.getGroupNumber(), list);
        }
        for (Integer groupNumber : map.keySet()) {
            MarkGroup group = groupDao.findOne(student.getExamId(), student.getSubjectCode(), groupNumber);
            if (group.getStatus() == MarkStatus.FINISH) {
                return false;
            }
        }
        for (Integer groupNumber : map.keySet()) {
            List<MarkLibrary> list = libraryDao.findByStudentIdAndGroupNumber(student.getId(), groupNumber);
            List<ExamQuestion> questions = questionDao.findByExamIdAndSubjectCodeAndObjectiveAndGroupNumber(
                    student.getExamId(), student.getSubjectCode(), false, groupNumber);
            List<MarkStepDTO> qList = map.get(groupNumber);
            MarkGroup group = groupDao.findOne(student.getExamId(), student.getSubjectCode(), groupNumber);
            int count = 0;
            for (MarkLibrary library : list) {
                if (library.getTaskNumber() == 3) {
                    // 出现三评时删除第三条任务
                    problemHistoryDao.deleteByLibraryId(library.getId());
                    libraryDao.delete(library);
                    updateLibraryCount(group);
                    count++;
                    continue;
                }
                if (library.getStatus().equals(LibraryStatus.ARBITRATED) || qList.size() == questions.size()) {
                    // 仲裁任务直接重置 或者该分组下所有分数被打回也重置
                    libraryDao.resetById(library.getId(), library.getMarkerId(), LibraryStatus.REJECTED,
                            LibraryStatus.MARKED, LibraryStatus.PROBLEM, LibraryStatus.INSPECTED,
                            LibraryStatus.ARBITRATED);
                    trackDao.deleteByLibraryId(library.getId());
                    specialTagDao.deleteByLibraryId(library.getId());
                    arbitrateDao.deleteByStudentIdAndGroupNumber(student.getId(), groupNumber);
                    count++;
                    continue;
                }
                if (library.getStatus().equals(LibraryStatus.MARKED)
                        || library.getStatus().equals(LibraryStatus.INSPECTED)) {
                    List<ScoreItem> sList = library.getScoreList();
                    for (MarkStepDTO markStepDTO : qList) {
                        trackDao.deleteByLibraryIdAndQuestionNumber(library.getId(), markStepDTO.getMainNumber() + "."
                                + markStepDTO.getSubNumber());
                        for (int i = 0; i < questions.size(); i++) {
                            ExamQuestion question = questions.get(i);
                            if (markStepDTO.getMainNumber() == question.getMainNumber()
                                    && markStepDTO.getSubNumber().equals(question.getSubNumber())) {
                                sList.remove(i);
                                sList.add(i, new ScoreItem(false));
                            }
                        }
                    }
                    StringBuilder markerScoreList = new StringBuilder();
                    for (int i = 0; i < sList.size(); i++) {
                        ScoreItem scoreItem = sList.get(i);
                        if (scoreItem.getScore() != null) {
                            markerScoreList.append(scoreItem.getScore());
                        } else {
                            markerScoreList.append("#");
                        }
                        if (i < sList.size() - 1) {
                            markerScoreList.append(",");
                        }
                    }
                    if (libraryDao
                            .updateMarkerResult(library.getId(), LibraryStatus.REJECTED, library.getMarkerId(), null,
                                    markerScoreList.toString(), null, null, LibraryStatus.MARKED,
                                    LibraryStatus.INSPECTED) == 1) {
                        count++;
                    }
                }
            }
            if (count > 0) {
                updateMarkedCount(group);
                resetStudentGroup(student.getId(), student.getExamId(), student.getSubjectCode(), groupNumber);
                studentService.updateSubjectiveStatusAndTimeAndInspectorId(student.getId(), SubjectiveStatus.UNMARK,
                        null, null);
            }
        }
        return true;
    }

    @Transactional
    @Override
    public boolean deleteByQuestion(ExamQuestion question) {
        if (!question.isObjective() && question.getGroupNumber() != null) {
            return false;
        } else {
            int examId = question.getExamId();
            String subjectCode = question.getSubjectCode();
            boolean objective = question.isObjective();
            questionDao.delete(question);
            if (objective) {
                examService.updateObjectiveStatus(examId, ObjectiveStatus.WAITING);
            } else {
                // 未分组的题目
                long unGroupQuestionCount = questionDao.countByExamIdAndSubjectAndObjectiveAndGroupNumberIsNull(examId,
                        subjectCode, false);
                // 考生整体状态与总分更新
                long groupCount = groupDao.countByExamIdAndSubjectCode(examId, subjectCode);
                if (groupCount == 0 || unGroupQuestionCount > 0) {
                    studentService.updateSubjectiveStatusAndScoreAndInspectorId(examId, subjectCode,
                            SubjectiveStatus.UNMARK, 0, null, null, null);
                } else {
                    List<Integer> studentList = studentService.findIdByExamIdAndSubjectCodeAndSubjectiveStatus(examId,
                            subjectCode, SubjectiveStatus.UNMARK, SubjectiveStatus.MARKED, SubjectiveStatus.INSPECTED);
                    for (Integer studentId : studentList) {
                        checkStudentSubjective(studentId, groupCount, unGroupQuestionCount);
                    }
                }
            }

            subjectService.updateScore(examId, subjectCode, objective,
                    questionService.sumTotalScore(examId, subjectCode, objective));

            return true;
        }
    }

    @Override
    @Transactional
    public boolean rejectLibrary(MarkLibrary library, MarkStepDTO[] questionList, Integer userId) {
        List<ExamQuestion> questions = questionDao.findByExamIdAndSubjectCodeAndObjectiveAndGroupNumber(
                library.getExamId(), library.getSubjectCode(), false, library.getGroupNumber());
        if (questionList.length == questions.size()) {
            return rejectLibrary(library, userId, false);
        }
        MarkGroup group = groupDao.findOne(library.getExamId(), library.getSubjectCode(), library.getGroupNumber());
        if (group.getStatus() == MarkStatus.FINISH) {
            return false;
        }
        List<ScoreItem> sList = library.getScoreList();
        for (MarkStepDTO markStepDTO : questionList) {
            trackDao.deleteByLibraryIdAndQuestionNumber(library.getId(), markStepDTO.getMainNumber() + "."
                    + markStepDTO.getSubNumber());
            for (int i = 0; i < questions.size(); i++) {
                ExamQuestion question = questions.get(i);
                if (markStepDTO.getMainNumber() == question.getMainNumber()
                        && markStepDTO.getSubNumber().equals(question.getSubNumber())) {
                    sList.remove(i);
                    sList.add(i, new ScoreItem(false));
                }
            }
        }
        StringBuilder markerScoreList = new StringBuilder();
        for (int i = 0; i < sList.size(); i++) {
            ScoreItem scoreItem = sList.get(i);
            if (scoreItem.getScore() != null) {
                markerScoreList.append(scoreItem.getScore());
            } else {
                markerScoreList.append("#");
            }
            if (i < sList.size() - 1) {
                markerScoreList.append(",");
            }
        }
        if (libraryDao.updateMarkerResult(library.getId(), LibraryStatus.REJECTED, library.getMarkerId(), null,
                markerScoreList.toString(), null, null, LibraryStatus.MARKED) == 1) {
            resetStudentGroup(library.getStudentId(), library.getExamId(), library.getSubjectCode(),
                    library.getGroupNumber());
            problemHistoryDao.resetByLibraryId(library.getId(), HistoryStatus.WAITING, userId, HistoryStatus.BACK,
                    new Date());
            // 开启三评时，打回1，2任务则删除第3条任务
            long count = libraryDao.countByStudentIdAndGroupNumber(library.getStudentId(), library.getGroupNumber());
            if (library.getTaskNumber() != 3 && count == 3) {
                MarkLibrary third = libraryDao.findByStudentIdAndGroupNumberAndTaskNumber(library.getStudentId(),
                        library.getGroupNumber(), 3);
                trackDao.deleteByLibraryId(third.getId());
                specialTagDao.deleteByLibraryId(third.getId());
                problemHistoryDao.deleteByLibraryId(third.getId());
                libraryDao.delete(third);
                updateLibraryCount(group);
            }
            updateMarkedCount(group);
            return true;
        } else {
            return false;
        }
    }
}
