package cn.com.qmth.stmms.admin.thread;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.biz.report.utils.ReportContext;
import cn.com.qmth.stmms.common.enums.ExamStatus;
import cn.com.qmth.stmms.common.enums.LockType;

public class ScoreReportThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(ScoreReportThread.class);

    private ExamStudentService studentService;

    private ExamQuestionService questionService;

    private int examId;

    private Set<String> subjectCodeSet;

    private LockService lockService;

    private ReportService reportService;

    private Map<String, List<ExamQuestion>> objectiveMap;

    private Map<String, List<ExamQuestion>> subjectiveMap;

    private Map<String, ExamSubject> subjectMap;

    private ExamSubjectService subjectService;

    private ExamService examService;

    private ReportContext context;

    private boolean examLock;

    public ScoreReportThread(int examId, Set<String> subjectCodeSet, LockService lockService,
            ExamStudentService studentService, ExamQuestionService questionService, ReportService reportService,
            ExamService examService, ExamSubjectService subjectService, boolean examLock) {
        this.examId = examId;
        this.subjectCodeSet = subjectCodeSet != null ? subjectCodeSet : new HashSet<>();
        this.lockService = lockService;
        this.studentService = studentService;
        this.questionService = questionService;
        this.reportService = reportService;
        this.examService = examService;
        this.subjectService = subjectService;
        this.examLock = examLock;
        this.objectiveMap = new HashMap<>();
        this.subjectiveMap = new HashMap<>();
        this.subjectMap = new HashMap<>();
    }

    @Override
    public void run() {
        // 获取考试信息并判断状态
        Exam exam = examService.findById(examId);
        if (exam == null || exam.getStatus() != ExamStatus.START) {
            log.info("report exception for examId=" + examId + ", exam is null or status error");
            return;
        }
        log.info("start report for examId=" + examId + ", subjectCode count=" + subjectCodeSet.size());

        if (examLock) {
            lockService.trylock(LockType.SCORE_CALCULATE, examId);
        }
        for (String subjectCode : subjectCodeSet) {
            // 尝试上锁，失败直接跳过
            if (!lockService.trylock(LockType.SCORE_CALCULATE, examId, subjectCode)) {
                log.warn("report locked for examId=" + examId + ", subjectCode=" + subjectCode);
                continue;
            }
            try {
                log.info("start report for examId=" + examId + ", subjectCode=" + subjectCode);
                // 删除原有统计数据
                reportService.deleteData(examId, subjectCode);
                context = new ReportContext(exam);
                int pageNumber = 1;
                int pageSize = 1000;
                List<ExamStudent> list = studentService.findByExamIdAndSubjectCode(examId, subjectCode, pageNumber,
                        pageSize);
                while (list != null && list.size() > 0) {
                    for (ExamStudent student : list) {
                        // 统计
                        statistic(student);
                    }
                    pageNumber++;
                    list = studentService.findByExamIdAndSubjectCode(examId, subjectCode, pageNumber, pageSize);
                }
                // 结束统计
                context.save();
            } catch (Exception e) {
                log.error("report exception for examId=" + examId + ", subjectCode=" + subjectCode, e);
            } finally {
                lockService.unlock(LockType.SCORE_CALCULATE, examId, subjectCode);
                log.info("finish report for examId=" + examId + ", subjectCode=" + subjectCode);
            }
        }
        if (examLock) {
            lockService.unlock(LockType.SCORE_CALCULATE, examId);
        }
    }

    private void statistic(ExamStudent student) {
        student.setSubject(findExamSubject(student.getSubjectCode()));
        student.setObjectiveQuestionList(findQuestionList(student.getSubjectCode(), student.getPaperType(), true));
        student.setSubjectiveQuestionList(findQuestionList(student.getSubjectCode(), student.getPaperType(), false));
        context.process(student);
    }

    private List<ExamQuestion> findQuestionList(String subjectCode, String paperType, boolean objective) {
        if (objective) {
            String key = subjectCode + "_" + StringUtils.trimToEmpty(paperType);
            List<ExamQuestion> list = objectiveMap.get(key);
            if (list == null) {
                list = questionService.findByExamAndSubjectAndObjectiveAndPaperType(examId, subjectCode, true,
                        paperType);
                objectiveMap.put(key, list);
            }
            return list;
        } else {
            List<ExamQuestion> list = subjectiveMap.get(subjectCode);
            if (list == null) {
                list = questionService.findByExamAndSubjectAndObjective(examId, subjectCode, false);
                subjectiveMap.put(subjectCode, list);
            }
            return list;
        }
    }

    private ExamSubject findExamSubject(String subjectCode) {
        ExamSubject subject = subjectMap.get(subjectCode);
        if (subject == null) {
            subject = subjectService.find(examId, subjectCode);
            subjectMap.put(subjectCode, subject);
        }
        return subject;
    }

}
